package utility;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import scenari.Automa;
import scenari.Scenari;

public class SubscribeCallback implements MqttCallback{

	private Scenari scenari;
//	private Publisher publisher;
	private Esecutore esec;
	private Automa automa;

	public SubscribeCallback(Scenari scenari, Esecutore esec, Automa automa) {
		this.scenari = scenari;
		this.esec = esec;
		this.automa = automa;
	}

	@Override
	public void connectionLost(Throwable arg0) {
		Helper.print("connessione persa\t"+arg0);
		final Date d = new Date();
		Date d2 = new Date();
		long time = Math.abs(d2.getTime()-d.getTime());

		while (time<600000) {
			try {
				Helper.print("retrying...");

				Scenari.main(null);
				return;
			} catch (MqttException | JSONException | IOException e) {
				Helper.print("non sono riusciuto");
				d2 = new Date();
				time = Math.abs(d2.getTime()-d.getTime());
			}
		}
		Helper.print("fuori dal while");
		if(time>=600000) {
			Helper.print("Tentativo di riconnessione fallito");
			System.exit(1);
		}
	}


	@Override
	public void deliveryComplete(IMqttDeliveryToken arg0) {
		// Nessuna operazione
		Helper.print("inviato");
	}


	@Override
	public void messageArrived(String topic, MqttMessage message) throws MqttException, JSONException, IOException {
//		topic = topic.toLowerCase();
		Helper.print("new message: "+topic+"  "+message.toString());
		if(topic.equals("to/all")) {
			Helper.print("invio lo stato dell'automa");
			sendRequested(Scenari.getMqttTree("from/","scenari/description"),false);
			return;
		}
		if(topic.equals(Scenari.getMqttTree("rpc/","scenari"))) {
			Helper.print("invio la mia configurazione");
			sendRequested(Scenari.getMqttTree("from/","scenari"),true);
			return;
		}

		JSONObject msgJson = new JSONObject(message.toString());
//		gli event sono 0 o 1; i trigger quando sono attivati/premuti inviano un 1, che e' l'unico elemento che mi interessa
		if(topic.equals(Scenari.getMqttTree("from/","gpio/"+scenari.getLearnTrigger()))) {
			Helper.print("trigger per learn\t"+!automa.getAntifurto());
			if(automa.getAntifurto())//se l'antifurto e' attivo, non posso attivare gli scenari
				return;
			int newStatus = msgJson.getInt("event");
			if(newStatus != 1)
				return;
			handleEvent(newStatus,scenari.getLearnTrigger());
			return;
		}
		if(topic.equals(Scenari.getMqttTree("from/","gpio/"+scenari.getAttivaScenari()))) {
			Helper.print("trigger per antifurto\t"+!automa.getLearn());
			if(automa.getLearn())//se l'apprendimento e' attivo, non posso attivare l'antifurto
				return;
			int newStatus = msgJson.getInt("event");
			if(newStatus != 1)
				return;
			handleEvent(newStatus,scenari.getAttivaScenari());
			return;
		}
		if(topic.startsWith(Scenari.getMqttTree("from/","gpio/IN"))) {
			Helper.print("totalmente inutile per me!");
			return;//non mi interessa altro da IN
		}
//		if(topic.equals("to/"+Scenari.getMqttTree()+"scenari/learn")) {//mi aspetto un {learn:true/false}, scritto da handleEvent
//			Helper.print("messaggio per iniziare/terminare a imparare");
//			boolean val = msgJson.getBoolean("learn");
//			handleLearn(val);
//			return;
//		}
		if(topic.startsWith(Scenari.getMqttTree("from/","gpio/OUT"))) {
			if(message.toString().contains("event")) {
				Helper.print("se learn==true, salvo questo event!");
				Helper.print(automa.getLearn()+"\t"+msgJson.toString());
			}
			if(!automa.getLearn() || !message.toString().contains("event"))
				return;
			if(scenarioName == null) {
				handleLearn(true);
			}
			int val = msgJson.getInt("event");
			String luce = topic.substring(23);
			handleLearnData(luce,val);
			return;
		}
		if(topic.equals(Scenari.getMqttTree("to/","scenari"))) {
			Helper.print("Cambio quale scenario e' attivo, se antifurto attivo");
			if(!automa.getAntifurto() || !message.toString().contains("event") || !message.toString().contains("nome"))
				return;
			changeActiveScenario(msgJson);
			return;
		}
		if(topic.equals(Scenari.getMqttTree("to/","scenari/salva"))) {
			Helper.print("rinomino l'ultimo scenario salvato");
			if(automa.ultimoStato != 1 || !message.toString().contains("nome"))//se non sono appena stato nello stato di registrazione, non serve andare avanti
				return;
			renameLastScenario(msgJson);
			return;
		}
		if(topic.equals(Scenari.getMqttTree("to","scenari/sensoreAntifurto"))){
			Helper.print("messaggio per cambiare il sensore che imposta l'antifurto");//serve se l'antifurto cambia sensore
			if(!message.toString().contains("attiva-scenari"))
				return;
			changeSensor(msgJson);
			return;
		}

		Helper.print("Impossibile");
	}

	private void changeSensor(JSONObject msgJson) throws JSONException, IOException {
		String newSensor = msgJson.getString("attiva-scenari");
		scenari.setAttivaScenari(newSensor);
		Helper.print("sensore cambiato in "+newSensor);
	}

	private void handleEvent(int newStatus,String which) throws JSONException, MqttException, IOException{
		String cosa = which == scenari.getLearnTrigger() ? "learn" : "antifurto";
		String cmd = automa.findPossibleCommand(cosa);
		Helper.print("Cosa: "+cosa+"\tCmd: "+cmd);
		if(cmd == null)
			return;
		Helper.print("handling event: "+cosa+"\t"+cmd);
		automa.setStato(new JSONObject().put(cosa,cmd).toString());
		Helper.print("stato\t"+automa.getStato());
		if(Helper.compareText(cosa, "learn")) {
			boolean val = Helper.compareText(cmd, "on");
			handleLearn(val);
			sendRequested(Scenari.getMqttTree("from/","scenari/learn"),false);
		} else
			sendRequested(Scenari.getMqttTree("from/","scenari/antifurto"),false);
	}

	private String scenarioName = null;

	private void handleLearn(boolean shouldLearn) throws JSONException, IOException{
		Helper.print("handleLearn "+shouldLearn+" ; "+automa.getLearn());
		automa.setStato("{\"learn\":"+ (shouldLearn ? "\"on\"" : "\"off\"")+"}");
		Helper.print("\t"+automa.getLearn());
		if(automa.getLearn()) {
			if(scenarioName == null) {
				scenarioName = "scenario-"+LocalDateTime.now().toString().replace(':', '-')+".json";
			}
		} else {
			scenarioName = null;
//			TODO tell antifurto to turn off if shouldLearn == true
		}
	}

	private void handleLearnData(String luce, int val) throws MqttException, IOException, JSONException {
		JSONObject j = new JSONObject();
//		tempo passato dallo scorso comando
		LocalTime now = LocalTime.now();
		j.put("tempo", now.format(DateTimeFormatter.ofPattern("HH-mm-ss")));
//		quale luce
		j.put("luce", luce);
//		accesa o spenta?
		j.put("cmd", val);
		Helper.appendiFile(j, esec.DB_FOLDER+scenarioName,false);
	}

	private void changeActiveScenario(JSONObject scenario) throws IOException, JSONException, MqttException {
		String nuovoScenario = esec.getUCS().requestToChange(scenario);
		Helper.print(nuovoScenario);
		JSONObject j = new JSONObject();
		j.put("nome", nuovoScenario);
		scenari.sendMqttMessage(Scenari.getMqttTree("from/","/scenari/attiva"), j.toString());
	}

	private void sendRequested(String topic,boolean moreInfo) throws MqttException, JSONException, IOException {
		JSONObject j = new JSONObject();
		j.put("stato", automa.getStato());
		if(moreInfo) {
			j.put("learn", automa.getLearn());
			boolean antifurto = automa.getAntifurto();
			j.put("antifurto", antifurto);
			j.put("learn-trigger", scenari.getLearnTrigger());
			j.put("attiva-scenari", scenari.getAttivaScenari());
//			invio tutti gli scenari
			{
				ArrayList<File> allScenariFile = esec.getAllScenariAvailable();
				JSONArray arr = new JSONArray();
				for(File f : allScenariFile) {
					JSONObject scenarioData = new JSONObject();
					String scenNome = esec.getScenarioNome(f);
					Helper.print(scenNome);
					scenarioData.put("nome", !esec.isFileRenamed(scenNome)? scenNome.substring(0,28) : esec.removeData(scenNome));
					scenarioData.put("data", esec.getScenarioData(f));
					arr.put(scenarioData);
				}
				j.put("scenari-disponibili", arr);
			}
//			invio lo scenario attivo
			if(antifurto) {
				String scenarioAttivo = esec.getScenarioAttivo();
				if(scenarioAttivo != null && !scenarioAttivo.equals("none")) {
					JSONObject scenarioValori = new JSONObject();
					scenarioValori.put("nome", !esec.isFileRenamed(scenarioAttivo) ? scenarioAttivo.substring(0,28) : esec.removeData(scenarioAttivo));
					scenarioValori.put("data", esec.getDataScenarioAttivo());
					j.put("scenario-attivo", scenarioValori);
				}
			}
		}
		scenari.sendMqttMessage(topic, j.toString());
	}

	private void renameLastScenario(JSONObject json) throws IOException, MqttException, JSONException {
		String nuovoNome = json.getString("nome");
		nuovoNome += nuovoNome.endsWith(".json") ? "" : ".json";
		ArrayList<File> available = esec.getAllScenariAvailable();
//		cerco l'ultimo scenario (sono nominati in base al giorno e all'ora, se non sono rinominati dall'utente)

		File max = available.get(0);
		String maxString = max.toString();
		for(File f : available) {
			String fString = f.toString();
			if(esec.isFileRenamed(f))//non mi interessano i file rinominati
				continue;
			if(esec.isFileRenamed(max) || fString.compareToIgnoreCase(maxString) > 0) {
				max = f;
				maxString = fString;
			}
		}
		String nome = esec.getScenarioNome(max);
		String[] path = max.toString().split("/");
		StringBuilder rename = new StringBuilder();
		for(int i = 0; i< path.length-1; i++)
			rename.append(path[i]).append('/');
		LocalDateTime now = LocalDateTime.now();
		String date = now.getYear()+"-"+(now.getMonthValue() < 10 ? "0"+now.getMonthValue() : now.getMonthValue())+"-"+(now.getDayOfMonth() < 10 ? "0"+now.getDayOfMonth() : now.getDayOfMonth());
		rename.append(esec.removeExtension(nuovoNome)).append("---").append(date).append(".json");

		boolean renamed = max.renameTo(new File(rename.toString()));

		JSONObject j = new JSONObject();
		String nuovoNomeSenzaEstensione = esec.removeData(esec.removeExtension(renamed ? nuovoNome : nome));
		j.put("nome", nuovoNomeSenzaEstensione);
		j.put("data", date);
		scenari.sendMqttMessage(Scenari.getMqttTree("from/","scenario/salva"), j.toString());
	}


}
